import Expression from '../../../expressions/expression';
import { EntitySchema } from '../../../schema';
import { Chaining } from '../../common';
import { QueryChainingKind } from './';
import { DynamoQueryAllResults } from './all-results';
import { DynamoQueryFilter } from './filter';
import { DynamoQueryPaginate } from './paginate';
import { DynamoQueryWithAttributes } from './with-attributes';
import { DynamoQueryWithOptions, WithOptions } from './with-options';
import { DynamoWithSortKeyCondition, SortKeyConditionOperator } from './with-sort-key-condition';
export declare type QueryType = 'query';
export interface Query<PartitionKey> {
    schema: EntitySchema;
    partitionKey: PartitionKey;
}
export declare class DynamoQuery<Entity, PartitionKey, SortKey, KeySchema> extends Chaining<QueryChainingKind> {
    protected _query: Query<PartitionKey>;
    constructor(query: Query<PartitionKey>);
    withSortKeyCondition(operator: SortKeyConditionOperator): DynamoWithSortKeyCondition<Entity, KeySchema>;
    filter(filterExpression: Expression): DynamoQueryFilter<Entity, KeySchema>;
    withAttributes<K extends keyof Entity>(attributes: K[]): DynamoQueryWithAttributes<Pick<Entity, K>, KeySchema>;
    withOptions(options: WithOptions): DynamoQueryWithOptions<Entity, KeySchema>;
    paginate(limit?: number, lastKey?: KeySchema): DynamoQueryPaginate<Entity, KeySchema>;
    allResults(): DynamoQueryAllResults<Entity, KeySchema>;
}
