"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../../common");
const all_results_1 = require("./all-results");
const filter_1 = require("./filter");
const paginate_1 = require("./paginate");
const with_attributes_1 = require("./with-attributes");
const with_options_1 = require("./with-options");
const with_sort_key_condition_1 = require("./with-sort-key-condition");
class DynamoQuery extends common_1.Chaining {
    constructor(query) {
        super('query');
        this._query = query;
        this._stack.push(this);
    }
    withSortKeyCondition(operator) {
        return new with_sort_key_condition_1.DynamoWithSortKeyCondition(this._query.schema, operator, this._stack);
    }
    filter(filterExpression) {
        return new filter_1.DynamoQueryFilter({ filterExpression }, this._stack);
    }
    withAttributes(attributes) {
        return new with_attributes_1.DynamoQueryWithAttributes(attributes, this._stack);
    }
    withOptions(options) {
        return new with_options_1.DynamoQueryWithOptions(this._stack, options);
    }
    paginate(limit, lastKey) {
        return new paginate_1.DynamoQueryPaginate(this._stack, { limit, lastKey });
    }
    allResults() {
        return new all_results_1.DynamoQueryAllResults(this._stack);
    }
}
exports.DynamoQuery = DynamoQuery;
