import Expression from '../../../expressions/expression';
import { BeginsWith, IsBetween, IsEqualTo, IsGreaterOrEqualTo, IsGreaterThan, IsLessOrEqualTo, IsLessThan } from '../../../expressions/operator';
import { EntitySchema } from '../../../schema';
import { Chaining } from '../../common';
import { QueryChainingKind } from './';
import { DynamoQueryAllResults } from './all-results';
import { DynamoQueryFilter } from './filter';
import { DynamoQueryPaginate } from './paginate';
import { DynamoQueryWithAttributes } from './with-attributes';
import { DynamoQueryWithOptions, WithOptions } from './with-options';
export declare type WithSortKeyConditionType = 'withSortKeyCondition';
export interface WithSortKeyCondition {
    expression: string;
    expressionAttributeNames: {
        [key: string]: string;
    };
    expressionAttributeValues: {
        [key: string]: {
            [key: string]: string | number;
        };
    };
}
export declare type SortKeyConditionOperator = BeginsWith | IsEqualTo | IsGreaterThan | IsLessThan | IsLessOrEqualTo | IsGreaterOrEqualTo | IsBetween;
export declare class DynamoWithSortKeyCondition<Entity, KeySchema> extends Chaining<QueryChainingKind> {
    private _withSortKeyCondition;
    constructor(schema: EntitySchema, operator: SortKeyConditionOperator, currentStack: Array<Chaining<QueryChainingKind>>);
    filter(filterExpression: Expression): DynamoQueryFilter<Entity, KeySchema>;
    withAttributes<K extends keyof Entity>(attributes: K[]): DynamoQueryWithAttributes<Pick<Entity, K>, KeySchema>;
    withOptions(options: WithOptions): DynamoQueryWithOptions<Entity, KeySchema>;
    paginate(limit?: number, lastKey?: KeySchema): DynamoQueryPaginate<Entity, KeySchema>;
    allResults(): DynamoQueryAllResults<Entity, KeySchema>;
    private buildSortKeyCondition(schema, sortKeyConditionOperator);
}
