"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const random_generator_1 = require("../../../expressions/random-generator");
const common_1 = require("../../common");
const all_results_1 = require("./all-results");
const filter_1 = require("./filter");
const paginate_1 = require("./paginate");
const with_attributes_1 = require("./with-attributes");
const with_options_1 = require("./with-options");
class DynamoWithSortKeyCondition extends common_1.Chaining {
    constructor(schema, operator, currentStack) {
        super('withSortKeyCondition', currentStack);
        this._withSortKeyCondition = this.buildSortKeyCondition(schema, operator);
        this._stack.push(this);
    }
    filter(filterExpression) {
        return new filter_1.DynamoQueryFilter({ filterExpression }, this._stack);
    }
    withAttributes(attributes) {
        return new with_attributes_1.DynamoQueryWithAttributes(attributes, this._stack);
    }
    withOptions(options) {
        return new with_options_1.DynamoQueryWithOptions(this._stack, options);
    }
    paginate(limit, lastKey) {
        return new paginate_1.DynamoQueryPaginate(this._stack, { limit, lastKey });
    }
    allResults() {
        return new all_results_1.DynamoQueryAllResults(this._stack);
    }
    buildSortKeyCondition(schema, sortKeyConditionOperator) {
        let expression;
        const sortKey = schema.tableSchema ? schema.tableSchema.sortKey : schema.indexSchema.sortKey;
        const randomId = '#' + random_generator_1.randomGenerator();
        if (sortKeyConditionOperator.type === 'function') {
            expression = sortKeyConditionOperator.value +
                `(${randomId},${sortKeyConditionOperator.functionOperand})`;
        }
        else {
            expression = randomId + ' ' + sortKeyConditionOperator.value;
        }
        const expressionAttributeNames = {
            [randomId]: sortKey,
        };
        const expressionAttributeValues = sortKeyConditionOperator.expressionAttributeValues;
        return { expression, expressionAttributeNames, expressionAttributeValues };
    }
}
exports.DynamoWithSortKeyCondition = DynamoWithSortKeyCondition;
