import { UpdateResult } from '../../database-operations/update';
import Expression from '../../expressions/expression';
import { ResolvedExpression } from '../../expressions/resolve-expression';
import { EntitySchema } from '../../schema';
import { Chaining } from '../common';
import { UpdateChainingKind } from './';
import { DynamoUpdateWithCondition } from './with-condition';
export declare type UpdateType = 'update';
export declare type ExplicitKeyItemType<Entity, KeySchema> = Partial<Entity>;
export declare type ImplicityKeyItemType<Entity, KeySchema> = Partial<Entity> & KeySchema;
export interface Update<KeySchema> {
    schema: EntitySchema;
    key: KeySchema;
    updateExpression: ResolvedExpression;
}
export interface UpdateWithExplicitKey<Entity, KeySchema> {
    schema: EntitySchema;
    key: KeySchema;
    item: ExplicitKeyItemType<Entity, KeySchema>;
}
export interface UpdateWithImplicitKey<Entity, KeySchema> {
    schema: EntitySchema;
    item: ImplicityKeyItemType<Entity, KeySchema>;
}
export declare class DynamoUpdate<Entity, KeySchema> extends Chaining<UpdateChainingKind> {
    protected _update: Update<KeySchema>;
    constructor(update: UpdateWithExplicitKey<Entity, KeySchema> | UpdateWithImplicitKey<Entity, KeySchema>);
    withCondition(expression: Expression): DynamoUpdateWithCondition<Entity, KeySchema>;
    execute(): Promise<UpdateResult<Entity>>;
    private buildUpdateWithExplicitKey(update);
    private buildUpdateWithImplicitKey(update);
}
