"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const random_generator_1 = require("../../expressions/random-generator");
const common_1 = require("../common");
const execute_1 = require("./execute");
const with_condition_1 = require("./with-condition");
const marshalItem = require('dynamodb-marshaler').marshalItem;
function isUpdateWithExplicitKey(update) {
    return update.key !== undefined;
}
function isUpdateWithImplicitKey(update) {
    return update.key === undefined;
}
class DynamoUpdate extends common_1.Chaining {
    constructor(update) {
        super('update');
        if (isUpdateWithExplicitKey(update)) {
            this._update = this.buildUpdateWithExplicitKey(update);
        }
        else if (isUpdateWithImplicitKey(update)) {
            this._update = this.buildUpdateWithImplicitKey(update);
        }
        this._stack.push(this);
    }
    withCondition(expression) {
        return new with_condition_1.DynamoUpdateWithCondition({ expression }, this._stack);
    }
    execute() {
        return execute_1.execute(this._stack);
    }
    buildUpdateWithExplicitKey(update) {
        let resolvedExpression = 'SET';
        let expressionAttributeNames = {};
        let expressionAttributeValues = {};
        if (!Object.keys(update.item).length) {
            throw new Error('UpdateEmptyItem');
        }
        for (const key in update.item) {
            if (update.item.hasOwnProperty(key)
                && update.schema.tableSchema.partitionKey !== key && update.schema.tableSchema.sortKey !== key) {
                const newName = `#${random_generator_1.randomGenerator()}`;
                const newValue = `:${random_generator_1.randomGenerator()}`;
                resolvedExpression += ` ${newName} = ${newValue},`;
                expressionAttributeNames = Object.assign({}, expressionAttributeNames, { [newName]: key });
                expressionAttributeValues = Object.assign(expressionAttributeValues, marshalItem({ [newValue]: update.item[key] }));
            }
        }
        resolvedExpression = resolvedExpression.slice(0, resolvedExpression.length - 1);
        return {
            schema: update.schema,
            key: update.key,
            updateExpression: { resolvedExpression, expressionAttributeNames, expressionAttributeValues },
        };
    }
    buildUpdateWithImplicitKey(update) {
        const tableSchema = update.schema.tableSchema;
        let resolvedExpression = 'SET';
        let expressionAttributeNames = {};
        let expressionAttributeValues = {};
        if (!Object.keys(update.item).length) {
            throw new Error('UpdateEmptyItem');
        }
        for (const prop in update.item) {
            if (update.item.hasOwnProperty(prop)
                && tableSchema.partitionKey !== prop && tableSchema.sortKey !== prop) {
                const newName = `#${random_generator_1.randomGenerator()}`;
                const newValue = `:${random_generator_1.randomGenerator()}`;
                resolvedExpression += ` ${newName} = ${newValue},`;
                expressionAttributeNames = Object.assign({}, expressionAttributeNames, { [newName]: prop });
                expressionAttributeValues = Object.assign(expressionAttributeValues, marshalItem({ [newValue]: update.item[prop] }));
            }
        }
        resolvedExpression = resolvedExpression.slice(0, resolvedExpression.length - 1);
        const key = {
            [tableSchema.partitionKey]: update.item[tableSchema.partitionKey],
        };
        if (tableSchema.sortKey) {
            key[tableSchema.sortKey] = update.item[tableSchema.sortKey];
        }
        return {
            schema: update.schema,
            key,
            updateExpression: { resolvedExpression, expressionAttributeNames, expressionAttributeValues },
        };
    }
}
exports.DynamoUpdate = DynamoUpdate;
