"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
const marshalItem = require('dynamodb-marshaler').marshalItem;
const BATCH_WRITE_LIMIT = 25; // items
const INITIAL_BACKOFF = 50; // ms
const MAX_BACKOFF = 1000; // ms
const segmentBatchDeleteInputs = (batchDeleteInput) => {
    const RequestItems = helpers_1.deepClone(batchDeleteInput.RequestItems);
    const tableName = Object.keys(RequestItems)[0];
    const segments = new Array();
    do {
        const segment = helpers_1.deepClone(batchDeleteInput);
        segment.RequestItems[tableName] = RequestItems[tableName].slice(0, BATCH_WRITE_LIMIT);
        segments.push(segment);
        if (RequestItems[tableName].length > BATCH_WRITE_LIMIT) {
            RequestItems[tableName] = RequestItems[tableName].slice(BATCH_WRITE_LIMIT, RequestItems[tableName].length);
        }
        else {
            RequestItems[tableName] = [];
        }
    } while (RequestItems[tableName].length);
    return segments;
};
const singleBatchDelete = (batchDeleteInput, dynamoPromise) => __awaiter(this, void 0, void 0, function* () {
    const tableName = Object.keys(batchDeleteInput.RequestItems)[0];
    try {
        const batchDeleteOutput = yield dynamoPromise.batchWrite(batchDeleteInput);
        if (batchDeleteOutput.UnprocessedItems && Object.keys(batchDeleteOutput.UnprocessedItems).length) {
            batchDeleteInput.RequestItems[tableName] =
                batchDeleteOutput.UnprocessedItems[tableName].map((request) => ({
                    DeleteRequest: { Key: marshalItem(request.DeleteRequest.Key) },
                }));
            return batchDeleteInput;
        }
        return undefined;
    }
    catch (err) {
        if (err.code === 'ProvisionedThroughputExceededException') {
            return batchDeleteInput;
        }
        else {
            throw new Error('UnknownError');
        }
    }
});
const mergeArray = (arr1, arr2) => {
    let arrMerge = new Array();
    arrMerge = arrMerge.concat(arr1);
    arrMerge = arrMerge.concat(arr2);
    return arrMerge;
};
function batchDelete(batchDeleteInput, dynamoPromise) {
    return __awaiter(this, void 0, void 0, function* () {
        let unprocessed = batchDeleteInput;
        const tableName = Object.keys(unprocessed.RequestItems)[0];
        do {
            const segments = segmentBatchDeleteInputs(unprocessed);
            const batchDeletes = segments.map((segment) => singleBatchDelete(segment, dynamoPromise));
            unprocessed = (yield Promise.all(batchDeletes)).reduce((acc, current) => {
                if (current) {
                    acc.RequestItems[tableName] = mergeArray(acc.RequestItems[tableName], current.RequestItems[tableName]);
                }
                return acc;
            }, { RequestItems: { [tableName]: [] } });
        } while (unprocessed.RequestItems[tableName].length);
        return {};
    });
}
exports.batchDelete = batchDelete;
