"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
const marshalItem = require('dynamodb-marshaler').marshalItem;
const BATCH_GET_LIMIT = 100; // items
const INITIAL_BACKOFF = 50; // ms
const MAX_BACKOFF = 1000; // ms
const segmentBatchGetInputs = (batchGetInput) => {
    const RequestItems = helpers_1.deepClone(batchGetInput.RequestItems);
    const tableName = Object.keys(RequestItems)[0];
    const segments = new Array();
    do {
        const segment = helpers_1.deepClone(batchGetInput);
        segment.RequestItems[tableName].Keys = RequestItems[tableName].Keys.slice(0, BATCH_GET_LIMIT);
        segments.push(segment);
        if (RequestItems[tableName].Keys.length > BATCH_GET_LIMIT) {
            RequestItems[tableName].Keys = RequestItems[tableName].Keys.slice(BATCH_GET_LIMIT, RequestItems[tableName].Keys.length);
        }
        else {
            RequestItems[tableName].Keys = [];
        }
    } while (RequestItems[tableName].Keys.length);
    return segments;
};
const singleBatchGet = (batchGetInput, dynamoPromise) => __awaiter(this, void 0, void 0, function* () {
    const tableName = Object.keys(batchGetInput.RequestItems)[0];
    try {
        const batchGetOutput = yield dynamoPromise.batchGet(batchGetInput);
        if (batchGetOutput.UnprocessedKeys && Object.keys(batchGetOutput.UnprocessedKeys).length) {
            batchGetInput.RequestItems[tableName].Keys =
                batchGetOutput.UnprocessedKeys[tableName].Keys.map((key) => (marshalItem(key)));
            return { batchGetOutput, batchGetInput };
        }
        return { batchGetOutput, batchGetInput: undefined };
    }
    catch (err) {
        if (err.code === 'ProvisionedThroughputExceededException') {
            return { batchGetInput, batchGetOutput: undefined };
        }
        else {
            throw new Error('UnknownError');
        }
    }
});
const mergeArray = (arr1, arr2) => {
    let arrMerge = new Array();
    arrMerge = arrMerge.concat(arr1);
    arrMerge = arrMerge.concat(arr2);
    return arrMerge;
};
const initalizeAcc = (tableName, batchGetInput) => {
    const clone = helpers_1.deepClone(batchGetInput);
    clone.RequestItems[tableName].Keys = [];
    return clone;
};
function batchGet(tableName, batchGetInput, dynamoPromise) {
    return __awaiter(this, void 0, void 0, function* () {
        let unprocessed = batchGetInput;
        let partial = new Array();
        do {
            const segments = segmentBatchGetInputs(unprocessed);
            const batchGets = segments.map((segment) => singleBatchGet(segment, dynamoPromise));
            unprocessed = (yield Promise.all(batchGets)).reduce((acc, current) => {
                if (current.batchGetInput) {
                    acc.RequestItems[tableName].Keys =
                        mergeArray(acc.RequestItems[tableName].Keys, current.batchGetInput.RequestItems[tableName].Keys);
                }
                if (current.batchGetOutput && current.batchGetOutput.Responses) {
                    partial = partial.concat(current.batchGetOutput.Responses[tableName]);
                }
                return acc;
            }, initalizeAcc(tableName, batchGetInput));
        } while (unprocessed.RequestItems[tableName].Keys.length);
        return { data: partial };
    });
}
exports.batchGet = batchGet;
