"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
const marshalItem = require('dynamodb-marshaler').marshalItem;
const BATCH_WRITE_LIMIT = 25; // items
const INITIAL_BACKOFF = 50; // ms
const MAX_BACKOFF = 1000; // ms
const segmentBatchWriteInputs = (batchWriteInput) => {
    const RequestItems = helpers_1.deepClone(batchWriteInput.RequestItems);
    const tableName = Object.keys(RequestItems)[0];
    const segments = new Array();
    do {
        const segment = helpers_1.deepClone(batchWriteInput);
        segment.RequestItems[tableName] = RequestItems[tableName].slice(0, BATCH_WRITE_LIMIT);
        segments.push(segment);
        if (RequestItems[tableName].length > BATCH_WRITE_LIMIT) {
            RequestItems[tableName] = RequestItems[tableName].slice(BATCH_WRITE_LIMIT, RequestItems[tableName].length);
        }
        else {
            RequestItems[tableName] = [];
        }
    } while (RequestItems[tableName].length);
    return segments;
};
const singleBatchWrite = (batchWriteInput, dynamoPromise) => __awaiter(this, void 0, void 0, function* () {
    const tableName = Object.keys(batchWriteInput.RequestItems)[0];
    try {
        const batchWriteOutput = yield dynamoPromise.batchWrite(batchWriteInput);
        if (batchWriteOutput.UnprocessedItems && Object.keys(batchWriteOutput.UnprocessedItems).length) {
            batchWriteInput.RequestItems[tableName] =
                batchWriteOutput.UnprocessedItems[tableName].map((request) => ({
                    PutRequest: { Item: marshalItem(request.PutRequest.Item) },
                }));
            return batchWriteInput;
        }
        return undefined;
    }
    catch (err) {
        if (err.code === 'ProvisionedThroughputExceededException') {
            return batchWriteInput;
        }
        else {
            throw new Error('UnknownError');
        }
    }
});
const mergeArray = (arr1, arr2) => {
    let arrMerge = new Array();
    arrMerge = arrMerge.concat(arr1);
    arrMerge = arrMerge.concat(arr2);
    return arrMerge;
};
function batchWrite(items, batchWriteInput, dynamoPromise) {
    return __awaiter(this, void 0, void 0, function* () {
        let unprocessed = batchWriteInput;
        const tableName = Object.keys(unprocessed.RequestItems)[0];
        do {
            const segments = segmentBatchWriteInputs(unprocessed);
            const batchWrites = segments.map((segment) => singleBatchWrite(segment, dynamoPromise));
            unprocessed = (yield Promise.all(batchWrites)).reduce((acc, current) => {
                if (current) {
                    acc.RequestItems[tableName] = mergeArray(acc.RequestItems[tableName], current.RequestItems[tableName]);
                }
                return acc;
            }, { RequestItems: { [tableName]: [] } });
        } while (unprocessed.RequestItems[tableName].length);
        return { data: items };
    });
}
exports.batchWrite = batchWrite;
