"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const random_generator_1 = require("../expressions/random-generator");
function buildExclusiveStartKey(lastKey) {
    let result = {};
    for (const propertyKey in lastKey) {
        if (lastKey.hasOwnProperty(propertyKey)) {
            result = Object.assign({}, result, {
                [propertyKey]: {
                    [typeof lastKey[propertyKey] === 'string' ? 'S' : 'N']: lastKey[propertyKey].toString(),
                },
            });
        }
    }
    return result;
}
exports.buildExclusiveStartKey = buildExclusiveStartKey;
function projectionExpression(attributes) {
    const result = attributes.reduce((acc, currentValue) => {
        return acc + currentValue + ',';
    }, '');
    return result.slice(0, result.length - 1);
}
exports.projectionExpression = projectionExpression;
function buildKey(key) {
    let result = {};
    for (const propertyKey in key) {
        if (key.hasOwnProperty(propertyKey)) {
            result = Object.assign({}, result, {
                [propertyKey]: {
                    [typeof key[propertyKey] === 'string' ? 'S' : 'N']: key[propertyKey].toString(),
                },
            });
        }
    }
    return result;
}
exports.buildKey = buildKey;
function mergeExpressionAttributeNames(...args) {
    let result = {};
    for (const expression of args) {
        if (expression) {
            result = Object.assign({}, result, expression);
        }
    }
    return Object.keys(result).length ? result : undefined;
}
exports.mergeExpressionAttributeNames = mergeExpressionAttributeNames;
function mergeExpressionAttributeValues(...args) {
    let result = {};
    for (const expression of args) {
        if (expression) {
            result = Object.assign({}, result, expression);
        }
    }
    return Object.keys(result).length ? result : undefined;
}
exports.mergeExpressionAttributeValues = mergeExpressionAttributeValues;
function buildKeyConditionExpression(partitionKey, withSortKeyCondition) {
    let keyConditionExpression = '';
    let expressionAttributeNames = {};
    let expressionAttributeValues = {};
    for (const partitionKeyName in partitionKey) {
        if (partitionKey.hasOwnProperty(partitionKeyName)) {
            const randomIdName = '#' + random_generator_1.randomGenerator();
            const randomIdValue = ':' + random_generator_1.randomGenerator();
            keyConditionExpression += `${randomIdName} = ${randomIdValue}`;
            expressionAttributeNames[randomIdName] = partitionKeyName;
            expressionAttributeValues[randomIdValue] = {
                [typeof partitionKey[partitionKeyName] === 'string' ? 'S' : 'N']: partitionKey[partitionKeyName],
            };
        }
    }
    if (withSortKeyCondition) {
        keyConditionExpression += ` AND ${withSortKeyCondition.expression}`;
        expressionAttributeNames = Object.assign({}, expressionAttributeNames, withSortKeyCondition.expressionAttributeNames);
        expressionAttributeValues = Object.assign({}, expressionAttributeValues, withSortKeyCondition.expressionAttributeValues);
    }
    return {
        keyConditionExpression,
        expressionAttributeNames,
        expressionAttributeValues,
    };
}
exports.buildKeyConditionExpression = buildKeyConditionExpression;
exports.deepClone = (obj) => JSON.parse(JSON.stringify(obj));
exports.timeoutPromise = (time) => setTimeout(() => new Promise((resolve, reject) => {
    resolve();
}), time);
