"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const resolve_expression_1 = require("../../expressions/resolve-expression");
const helpers_1 = require("../helpers");
const constants_1 = require("./constants");
function buildQueryInput(query, withSortKeyCondition, filter, withAttributes, withOptions, paginate) {
    const input = {
        TableName: query.schema.tableName,
    };
    const keyConditionExpression = helpers_1.buildKeyConditionExpression(query.partitionKey, withSortKeyCondition);
    input.KeyConditionExpression = keyConditionExpression.keyConditionExpression;
    let resolvedFilterExpression;
    if (query.schema.indexSchema) {
        input.IndexName = query.schema.indexSchema.indexName;
    }
    if (filter) {
        resolvedFilterExpression = resolve_expression_1.resolveExpression(filter.filterExpression.stack);
        input.FilterExpression = resolvedFilterExpression.resolvedExpression;
    }
    if (withAttributes) {
        input.ProjectionExpression = helpers_1.projectionExpression(withAttributes.attributes);
    }
    if (withOptions) {
        input.ScanIndexForward = withOptions.order === 'descending' ? false : true;
    }
    if (paginate) {
        input.Limit = paginate.limit || constants_1.DEFAULT_QUERY_LIMIT;
        if (paginate.lastKey) {
            input.ExclusiveStartKey = helpers_1.buildExclusiveStartKey(paginate.lastKey);
        }
    }
    input.ExpressionAttributeNames = helpers_1.mergeExpressionAttributeNames(keyConditionExpression.expressionAttributeNames, withSortKeyCondition && withSortKeyCondition.expressionAttributeNames, resolvedFilterExpression && resolvedFilterExpression.expressionAttributeNames, withAttributes && withAttributes.expressionAttributeNames);
    input.ExpressionAttributeValues = helpers_1.mergeExpressionAttributeValues(keyConditionExpression.expressionAttributeValues, withSortKeyCondition && withSortKeyCondition.expressionAttributeValues, resolvedFilterExpression && resolvedFilterExpression.expressionAttributeValues);
    return input;
}
exports.buildQueryInput = buildQueryInput;
