"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const resolve_expression_1 = require("../../expressions/resolve-expression");
const helpers_1 = require("../helpers");
const constants_1 = require("./constants");
function buildScanInput(entitySchema, filter, withAttributes, paginate) {
    const input = {
        TableName: entitySchema.tableName,
    };
    let resolvedFilterExpression;
    if (entitySchema.indexSchema) {
        input.IndexName = entitySchema.indexSchema.indexName;
    }
    if (filter) {
        resolvedFilterExpression = resolve_expression_1.resolveExpression(filter.filterExpression.stack);
        input.FilterExpression = resolvedFilterExpression.resolvedExpression;
    }
    if (withAttributes) {
        input.ProjectionExpression = helpers_1.projectionExpression(withAttributes.attributes);
    }
    if (paginate) {
        input.Limit = paginate.limit || constants_1.DEFAULT_SCAN_LIMIT;
        if (paginate.lastKey) {
            input.ExclusiveStartKey = helpers_1.buildExclusiveStartKey(paginate.lastKey);
        }
    }
    if (filter || withAttributes) {
        input.ExpressionAttributeNames = helpers_1.mergeExpressionAttributeNames(resolvedFilterExpression && resolvedFilterExpression.expressionAttributeNames, withAttributes && withAttributes.expressionAttributeNames);
        input.ExpressionAttributeValues = helpers_1.mergeExpressionAttributeValues(resolvedFilterExpression && resolvedFilterExpression.expressionAttributeValues);
    }
    return input;
}
exports.buildScanInput = buildScanInput;
