"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const condition_1 = require("./condition");
const expression_1 = require("./expression");
const match_1 = require("./match");
const not_1 = require("./not");
const random_generator_1 = require("./random-generator");
class AndExpression extends expression_1.default {
    constructor(currentStack) {
        super('and', currentStack);
        this.stack.push(this);
    }
    condition(expression) {
        return new condition_1.default(expression, this.stack);
    }
    not(expression) {
        return new not_1.default(expression, this.stack);
    }
    match(operand, operator) {
        const randomId = '#' + random_generator_1.randomGenerator();
        const expressionAttributeNames = {
            [randomId]: operand,
        };
        if (operator.type === 'function') {
            return new match_1.default(operator.value + `(${randomId},${operator.functionOperand})`, expressionAttributeNames, operator.expressionAttributeValues, this.stack);
        }
        else {
            return new match_1.default(randomId + ' ' + operator.value, expressionAttributeNames, operator.expressionAttributeValues, this.stack);
        }
    }
    attributeExists(attribute) {
        const randomId = '#' + random_generator_1.randomGenerator();
        const expressionAttributeNames = {
            [randomId]: attribute,
        };
        return new match_1.default(`attribute_exists(${randomId})`, expressionAttributeNames, undefined, this.stack);
    }
    attributeNotExists(attribute) {
        const randomId = '#' + random_generator_1.randomGenerator();
        const expressionAttributeNames = {
            [randomId]: attribute,
        };
        return new match_1.default(`attribute_not_exists(${randomId})`, expressionAttributeNames, undefined, this.stack);
    }
    size(attribute, operator) {
        const randomId = '#' + random_generator_1.randomGenerator();
        const expressionAttributeNames = {
            [randomId]: randomId,
        };
        return new match_1.default(`size(${randomId}) ${operator.value}`, expressionAttributeNames, operator.expressionAttributeValues, this.stack);
    }
}
exports.default = AndExpression;
