"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const condition_1 = require("./condition");
const match_1 = require("./match");
const not_1 = require("./not");
const random_generator_1 = require("./random-generator");
function condition(expression) {
    return new condition_1.default(expression);
}
exports.condition = condition;
function not(expression) {
    return new not_1.default(expression);
}
exports.not = not;
function match(operand, operator) {
    const randomId = '#' + random_generator_1.randomGenerator();
    const expressionAttributeNames = {
        [randomId]: operand,
    };
    if (operator.type === 'function') {
        return new match_1.default(operator.value + `(${randomId},${operator.functionOperand})`, expressionAttributeNames, operator.expressionAttributeValues);
    }
    else {
        return new match_1.default(randomId + ' ' + operator.value, expressionAttributeNames, operator.expressionAttributeValues);
    }
}
exports.match = match;
function attributeExists(attribute) {
    const randomId = '#' + random_generator_1.randomGenerator();
    const expressionAttributeNames = {
        [randomId]: attribute,
    };
    return new match_1.default(`attribute_exists(${randomId})`, expressionAttributeNames);
}
exports.attributeExists = attributeExists;
function attributeNotExists(attribute) {
    const randomId = '#' + random_generator_1.randomGenerator();
    const expressionAttributeNames = {
        [randomId]: attribute,
    };
    return new match_1.default(`attribute_not_exists(${randomId})`, expressionAttributeNames);
}
exports.attributeNotExists = attributeNotExists;
function size(attribute, operator) {
    const randomId = '#' + random_generator_1.randomGenerator();
    const expressionAttributeNames = {
        [randomId]: attribute,
    };
    return new match_1.default(`size(${randomId}) ${operator.value}`, expressionAttributeNames, operator.expressionAttributeValues);
}
exports.size = size;
function beginsWith(subString) {
    const randomId = ':' + random_generator_1.randomGenerator();
    return {
        value: 'begins_with',
        functionOperand: randomId,
        type: 'function',
        expressionAttributeValues: {
            [randomId]: {
                ['S']: subString,
            },
        },
        kind: 'beginsWith',
    };
}
exports.beginsWith = beginsWith;
function contains(operand) {
    const randomId = ':' + random_generator_1.randomGenerator();
    return {
        value: 'contains',
        functionOperand: randomId,
        type: 'function',
        expressionAttributeValues: {
            [randomId]: {
                ['S']: operand,
            },
        },
        kind: 'contains',
    };
}
exports.contains = contains;
function isEqualTo(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    let type;
    switch (typeof a) {
        case 'boolean': {
            type = 'BOOL';
            break;
        }
        case 'number': {
            type = 'N';
            break;
        }
        default: type = 'S';
    }
    return {
        value: `= ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isEqualTo',
    };
}
exports.isEqualTo = isEqualTo;
function isNotEqualTo(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    let type;
    switch (typeof a) {
        case 'boolean': {
            type = 'BOOL';
            break;
        }
        case 'number': {
            type = 'N';
            break;
        }
        default: type = 'S';
    }
    return {
        value: `<> ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isNotEqualTo',
    };
}
exports.isNotEqualTo = isNotEqualTo;
function isGreaterThan(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    const type = typeof a === 'string' ? 'S' : 'N';
    return {
        value: `> ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isGreaterThan',
    };
}
exports.isGreaterThan = isGreaterThan;
function isLessThan(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    const type = typeof a === 'string' ? 'S' : 'N';
    return {
        value: `< ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isLessThan',
    };
}
exports.isLessThan = isLessThan;
function isLessOrEqualTo(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    const type = typeof a === 'string' ? 'S' : 'N';
    return {
        value: `<= ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isLessOrEqualTo',
    };
}
exports.isLessOrEqualTo = isLessOrEqualTo;
function isGreaterOrEqualTo(a) {
    const randomId = ':' + random_generator_1.randomGenerator();
    const type = typeof a === 'string' ? 'S' : 'N';
    return {
        value: `>= ${randomId}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId]: {
                [type]: a.toString(),
            },
        },
        kind: 'isGreaterOrEqualTo',
    };
}
exports.isGreaterOrEqualTo = isGreaterOrEqualTo;
function isBetween(b, c) {
    const randomId1 = ':' + random_generator_1.randomGenerator();
    const randomId2 = ':' + random_generator_1.randomGenerator();
    return {
        value: `BETWEEN ${randomId1} AND ${randomId2}`,
        type: 'comparison',
        expressionAttributeValues: {
            [randomId1]: {
                ['N']: b.toString(),
            },
            [randomId2]: {
                ['N']: c.toString(),
            },
        },
        kind: 'isBetween',
    };
}
exports.isBetween = isBetween;
function isIn(values) {
    const randomIds = new Array();
    const expressionAttributeValues = values.reduce((acc, value) => {
        const randomId = ':' + random_generator_1.randomGenerator();
        acc[randomId] = {
            ['S']: value,
        };
        randomIds.push(randomId);
        return acc;
    }, {});
    const result = randomIds.reduce((acc, value) => {
        return acc + value.toString() + ', ';
    }, `IN (`);
    return {
        value: result.slice(0, result.length - 2) + ')',
        type: 'comparison',
        expressionAttributeValues,
        kind: 'isIn',
    };
}
exports.isIn = isIn;
