"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const delete_1 = require("../../chaining/delete");
const find_1 = require("../../chaining/find");
const save_1 = require("../../chaining/save");
const update_1 = require("../../chaining/update");
const dynamo_entity_1 = require("../dynamo-entity");
class DynamoTableWithCompositeKey extends dynamo_entity_1.DynamoEntityWithCompositeKey {
    constructor(tableSchema, dynamoPromise) {
        super({
            tableName: tableSchema.tableName,
            tableSchema,
            dynamoPromise,
        });
    }
    find(args) {
        if (!args) {
            return new find_1.DynamoScan(this._entitySchema);
        }
        if (args.constructor === Array && args.length) {
            return new find_1.DynamoBatchGet({
                schema: this._entitySchema, keys: args,
            });
        }
        if (args.constructor === Array && args.length === 0) {
            throw new Error('BatchGetWithNoKeys');
        }
        if (Object.keys(args).length === 2) {
            return new find_1.DynamoGet({ schema: this._entitySchema, key: args });
        }
        else {
            return new find_1.DynamoQuery({
                schema: this._entitySchema, partitionKey: args,
            });
        }
    }
    save(args) {
        if (args.constructor !== Array) {
            return new save_1.DynamoPut({
                schema: this._entitySchema,
                item: args,
            });
        }
        if (args.length) {
            return new save_1.DynamoBatchWrite({
                schema: this._entitySchema,
                items: args,
            });
        }
        else {
            throw new Error('BatchWriteWithNoItems');
        }
    }
    update(...args) {
        if (args.length === 2) {
            return new update_1.DynamoUpdate({
                schema: this._entitySchema,
                key: args[0],
                item: args[1],
            });
        }
        else {
            return new update_1.DynamoUpdate({
                schema: this._entitySchema,
                item: args[0],
            });
        }
    }
    delete(args) {
        if (args.constructor !== Array) {
            return new delete_1.DynamoDelete({
                schema: this._entitySchema,
                key: args,
            });
        }
        else {
            if (!args.length) {
                throw new Error('BatchDeleteWithNoKeys');
            }
            return new delete_1.DynamoBatchDelete({
                schema: this._entitySchema,
                keys: args,
            });
        }
    }
}
exports.DynamoTableWithCompositeKey = DynamoTableWithCompositeKey;
