import DynamoPromise from '../../../../database-operations/dynamo-to-promise';
import { DynamoIndexWithCompositeKey, DynamoIndexWithSimpleKey } from '../../../../schema/dynamo-index';
import { DynamoTableWithCompositeKey } from '../../../../schema/dynamo-table';
import { CompositeKeyWithGlobalIndex } from './with-global-index';
export declare class TypeDynamoDefineTableCompositeKey<Table, PartitionKey extends keyof Table, SortKey extends keyof Table> {
    private dynamoPromise;
    private tableSchema;
    constructor(dynamoPromise: DynamoPromise, schema: {
        tableName: string;
        partitionKey: PartitionKey;
        sortKey: SortKey;
    });
    getInstance(): DynamoTableWithCompositeKey<Table, Pick<Table, PartitionKey>, Pick<Table, SortKey>>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'KEYS_ONLY';
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Pick<Table, IndexPartitionKey> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, {}>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'KEYS_ONLY';
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Pick<Table, IndexPartitionKey | IndexSortKey> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, {}>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'ALL';
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Table, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, {}>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'ALL';
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Table, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, {}>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexAttributes extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'INCLUDE';
        attributes: IndexAttributes[];
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Pick<Table, IndexPartitionKey | IndexAttributes> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, {}>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table, IndexAttributes extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'INCLUDE';
        attributes: IndexAttributes[];
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, {} & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Pick<Table, IndexPartitionKey | IndexSortKey | IndexAttributes> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, {}>;
    private buildTableSchema(schema);
}
