"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dynamo_index_1 = require("../../../../schema/dynamo-index");
const dynamo_table_1 = require("../../../../schema/dynamo-table");
const with_global_index_1 = require("./with-global-index");
class TypeDynamoDefineTableCompositeKey {
    constructor(dynamoPromise, schema) {
        this.dynamoPromise = dynamoPromise;
        this.tableSchema = this.buildTableSchema(schema);
    }
    getInstance() {
        return new dynamo_table_1.DynamoTableWithCompositeKey(this.tableSchema, this.dynamoPromise);
    }
    withGlobalIndex(config) {
        if (config.sortKey) {
            return new with_global_index_1.CompositeKeyWithGlobalIndex(this.dynamoPromise, this.tableSchema, { [config.indexName]: new dynamo_index_1.DynamoIndexWithCompositeKey(config) }, {});
        }
        return new with_global_index_1.CompositeKeyWithGlobalIndex(this.dynamoPromise, this.tableSchema, { [config.indexName]: new dynamo_index_1.DynamoIndexWithSimpleKey(config) }, {});
    }
    buildTableSchema(schema) {
        const tableSchema = {
            tableName: schema.tableName,
            partitionKey: schema.partitionKey,
            sortKey: schema.sortKey,
            writeCapacity: schema.writeCapacity || 1,
            readCapacity: schema.readCapacity || 1,
        };
        return tableSchema;
    }
}
exports.TypeDynamoDefineTableCompositeKey = TypeDynamoDefineTableCompositeKey;
