import DynamoPromise from '../../../../database-operations/dynamo-to-promise';
import { TableSchema } from '../../../../schema';
import { DynamoIndexWithCompositeKey, DynamoIndexWithSimpleKey } from '../../../../schema/dynamo-index';
import { DynamoORMWithCompositeKey } from '../../../../schema/dynamo-orm';
export declare class CompositeKeyWithGlobalIndex<Table, PartitionKey extends keyof Table, SortKey extends keyof Table, CurrentGlobalIndexes, CurrentLocalIndexes> {
    private dynamoPromise;
    private tableSchema;
    private currentGlobalIndexes;
    private currentLocalIndexes;
    constructor(dynamoPromise: DynamoPromise, tableSchema: TableSchema, currentGlobalIndexes: CurrentGlobalIndexes, currentLocalIndexes: CurrentLocalIndexes);
    getInstance(): DynamoORMWithCompositeKey<Table, Pick<Table, PartitionKey>, Pick<Table, SortKey>, CurrentGlobalIndexes, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'KEYS_ONLY';
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Pick<Table, IndexPartitionKey> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'KEYS_ONLY';
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Pick<Table, IndexPartitionKey | IndexSortKey> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'ALL';
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Table, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'ALL';
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Table, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexAttributes extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'INCLUDE';
        attributes: IndexAttributes[];
        partitionKey: IndexPartitionKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithSimpleKey<Pick<Table, IndexPartitionKey | IndexAttributes> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey>>;
    }, CurrentLocalIndexes>;
    withGlobalIndex<IndexName extends string, IndexPartitionKey extends keyof Table, IndexSortKey extends keyof Table, IndexAttributes extends keyof Table>(config: {
        indexName: IndexName;
        projectionType: 'INCLUDE';
        attributes: IndexAttributes[];
        partitionKey: IndexPartitionKey;
        sortKey: IndexSortKey;
    }): CompositeKeyWithGlobalIndex<Table, PartitionKey, SortKey, CurrentGlobalIndexes & {
        [P in IndexName]: DynamoIndexWithCompositeKey<Pick<Table, IndexPartitionKey | IndexSortKey | IndexAttributes> & Pick<Table, PartitionKey | SortKey>, Pick<Table, IndexPartitionKey>, Pick<Table, IndexSortKey>, Pick<Table, PartitionKey | SortKey> & Pick<Table, IndexPartitionKey | IndexSortKey>>;
    }, CurrentLocalIndexes>;
}
